import os
import csv
import sys
import math
import numpy as np
import pandas as pd
from scipy import signal
import logging
import networkx
import matplotlib.pyplot as plt
from pandas.plotting import autocorrelation_plot
from random import *


np.seterr(divide='ignore', invalid='ignore')
# Suppressing Warning
import warnings
import matplotlib.cbook
warnings.filterwarnings("ignore",category=matplotlib.cbook.mplDeprecation)

# --------------------- Functions---------------------
# AutoCorrelation WK Theorem
def Correlation(Matrix):
    
    df2 = Matrix
    mean = np.mean(df2) #np.mean(np.mean(df2))
    df3 = np.subtract(df2, mean)
    summing = np.sum(np.power(df3, 2))#np.sum(np.sum(np.power(df3, 2)))
    normalize_coeff = np.sqrt(summing)
    df4 = np.true_divide(df3, normalize_coeff)
    fftI = np.fft.fft2(df4)
    inner = np.fft.ifft2(np.multiply(fftI, fftI.conjugate()))
    shifting = np.fft.fftshift(inner)
    CorMatrix = shifting.real

    return CorMatrix

def CorArray(Array):
    df2 = Array
    mean = np.mean(df2)
    df3 = np.subtract(df2, mean)
    summing = np.sum(np.sum(np.power(df3, 2)))
    normalize_coeff = np.sqrt(summing)
    df4 = np.true_divide(df3, normalize_coeff)
    fftI = np.fft.fftn(df4)
    inner = np.fft.ifftn(np.multiply(fftI, fftI.conjugate()))
    # shifting = np.fft.fftshift(inner)
    CorrArray = inner.real #shifting.real
    AdjMatrix = np.absolute(CorrArray)

    return AdjMatrix


SZ = 100

# Matrix with only 1s 
def funONE():
    one = np.random.normal(1, 0.1, size=(SZ, SZ))
    return one

# Matrix with only 0s 
def funZERO():
    zero = np.random.normal(0, 0.1, size=(SZ, SZ))
    return zero

#  Matrix with 1s and 0s alternating along quadrants 
def funALTQ():
    altq = np.random.normal(0, 0.1, size=(SZ,SZ))
    SubOne = np.random.normal(1, 0.1, size=(int(SZ/2), int(SZ/2)))
    altq[int(SZ/2):SZ, int(SZ/2):SZ] = SubOne
    altq[0:int(SZ/2), 0:int(SZ/2)] = SubOne
    return altq

# Next two functions: for time correlation
def serial_corr(wave, lag = 1):
    n = len(wave)
    y1 = wave[lag:]
    y2 = wave[:n-lag]        
    corr = np.corrcoef(y1, y2)[0, 1]
    return corr

def autoCorr(wave):
    lags = range(len(wave)//2)
    corrs = [serial_corr(wave,lag) for lag in lags]
    return lags,corrs

# A = np.matrix([[2, 2, 2, 2, 2], 
            #    [2, 1, 1, 1, 2],
            #    [2, 1, 5, 1, 2],
            #    [2, 1, 1, 1, 2],
            #    [2, 2, 2, 2, 2]])
def RadialAverage(A):
  
    length = math.ceil(len(A)/2) #ceil is present in case it is a odd matrix
    RadialAvg = []

    for i in range(1, length):
        # Selecting outer most elements of matrix and calculating mean
        C = A[:,[0,len(A)-1]].ravel()
        R = A[[0,len(A)-1],:].ravel()
        Array = np.concatenate([R,C])
        MEAN = np.mean(Array)

        # Adding mean to a vector
        RadialAvg.append(MEAN)
        A = A[1:-1, 1:-1]

    # ----- only if odd matrix, enable this -----
    # RadialAvg.append(A.mean())

    RadialAvg = RadialAvg[::-1]

    return RadialAvg

# Histogram
def Hist(M):
    plt.hist(M, bins=50,color='#0504aa')
    plt.title("Temporal Distribution")
    plt.xlabel("Values")
    plt.ylabel("Counts")
    plt.show()

# Scatter Plot
def ScatterPlot(N,M):
    plt.figure()
    plt.scatter(N, M,s=10, marker='o',color='#0504aa'),
    plt.title("Autocorrelation Function")
    plt.xlabel("lags(r)")
    plt.ylabel("N(r)")
    plt.show()

def LogScatter(N,M):
    plt.semilogx(N,M)
    plt.show()

# HeatMap
def HeatMap(CorM):
    plt.matshow(CorM)
    plt.colorbar()
    plt.show()

def HistHeatMap(HHM):
    Array = np.asarray(HHM).reshape(-1)    
    hist = Hist(Array)   
    CorMatrix = Correlation(HHM)
    HM = HeatMap(CorMatrix)

    return hist, HM    

# ------------------ First Case : 1s and 0s matrix alternate - TIME IS BROKEN, SPACE IS CORRELATED ------------------
# ******************************** TIME ********************************
flag = 0
data1 = []

for i in range(1, 1000):

    if (flag == 0):

        one = funONE()
        MEAN = np.mean(one)
        data1.append(MEAN)
        flag = 1

    else:   
        zero = funZERO()
        MEAN = np.mean(zero)
        data1.append(MEAN)
        flag = 0

N=np.arange(1,1000)
serial_corr(data1, lag =1)
lags,corrs = autoCorr(data1)

#UNCOMMENT TO ENABLE

# Hist(data1)
# ScatterPlot(lags, corrs) 

# ------------------ Second Case : Altq 1000 times - SPACE IS BROKEN, TIME IS CORRELATED ------------------
# ******************************** TIME ********************************
data2 = []

for i in range(1, 1000):
    altq = funALTQ()
    MEAN = np.mean(altq)
    data2.append(MEAN)

serial_corr(data2, lag =1)
lags,corrs = autoCorr(data2)

#UNCOMMENT TO ENABLE

# Hist(data2) 
# ScatterPlot(lags, corrs) 

# ******************************** SPACE ********************************

# ******************************** SPACE ********************************

# ----- 1 and 0 matrix -----
one = funONE()
zero = funZERO()

# HistHeatMap(one) #UNCOMMENT TO ENABLE
# HistHeatMap(zero) #UNCOMMENT TO ENABLE

# ----- 1s and 0s alternating matrix -----
altq = funALTQ()
# HistHeatMap(altq) #UNCOMMENT TO ENABLE

RArray = RadialAverage(altq) # Change to zero, one or altq
SPACE = CorArray(RArray)
LEN = len(SPACE)

N=np.arange(1,LEN+1)
# ScatterPlot(N,SPACE)
# Hist(RArray)

x1= np.random.normal(0,0.1, size=(500,))
x2= np.random.normal(1,0.1, size=(500,))

alting=np.concatenate((x1, x2))
Hist(alting)